import { MigrationInterface, QueryRunner, Table, TableForeignKey } from "typeorm";
import { DB_TYPE } from "../env.defaults";

export class AddApiTokens1737453096674 implements MigrationInterface {
  name = "AddApiTokens1737453096674";

  public async up(queryRunner: QueryRunner): Promise<void> {
    const variableType_int = DB_TYPE == "mysql" ? "int" : "integer";

    await queryRunner.createTable(
      new Table({
        name: "api",
        columns: [
          { name: "token", type: "varchar", length: "255", isPrimary: true, isNullable: false },
          { name: "title", type: "varchar", length: "255", isNullable: false },
          { name: "createdAt", type: "datetime", default: "CURRENT_TIMESTAMP(6)", isNullable: false },
          { name: "lastUsage", type: "datetime", isNullable: true },
          { name: "expiry", type: "datetime", isNullable: true },
        ],
      }),
      true
    );

    await queryRunner.createTable(
      new Table({
        name: "api_permission",
        columns: [
          { name: "apiToken", type: "varchar", length: "255", isPrimary: true, isNullable: false },
          { name: "permission", type: "varchar", length: "255", isPrimary: true, isNullable: false },
        ],
      })
    );

    await queryRunner.createForeignKey(
      "api_permission",
      new TableForeignKey({
        columnNames: ["apiToken"],
        referencedColumnNames: ["token"],
        referencedTableName: "api",
        onDelete: "CASCADE",
        onUpdate: "RESTRICT",
      })
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    const table = await queryRunner.getTable("api_permission");
    const foreignKey = table.foreignKeys.find((fk) => fk.columnNames.indexOf("apiToken") !== -1);
    await queryRunner.dropForeignKey("api_permission", foreignKey);
    await queryRunner.dropTable("api_permission");
    await queryRunner.dropTable("api");
  }
}
